<?php
	header('Content-Type: text/html; charset=utf-8');
	$is_local = $_SERVER['HTTP_HOST'] == 'localhost';
	
	if ($is_local) error_reporting(E_ALL);
	
	$perl = true;
	$python = true;
	$ruby = $is_local;
	$java = $is_local;
	$lua = $is_local;
	$pandoc = $is_local;
	
	$normalize = $is_local && isset($_GET['normalize']);
?>
<!DOCTYPE html>

<html xmlns="http://www.w3.org/1999/xhtml" lang="en">
<head>
	<title>Babelmark — Markdown Testbed</title>
	<link rel="stylesheet" type="text/css" href="/style/dingus.css" media="all, handheld" />
	<style>
		p { text-align: left ! important }
		#titre p { margin-right: 1em }
	</style>
</head>
<body>

<div id="titre">
	<h1>Babelmark<br /><i style="font-size:90%">Markdown Testbed</i></h1>
	
	<p>
	Babelmark is a testbed for side-by-side comparaisons of the output 
	of various Markdown implementations.
	</p>
</div>

<div id="app">
	<form method="get" action="./">
	<div>
		<label for="markdown">Text Source:</label><br />
		<textarea name="markdown" id="markdown" rows="10" cols="80"><?php
			if (isset($_GET['markdown']))
				echo htmlentities(stripslashes($_GET['markdown']), 
												ENT_NOQUOTES, 'UTF-8');
		?></textarea>
	</div>

	<div id="buttonrow">
<?php if ($is_local) : ?>
		<label for="normalize" style="margin-right:2em">
			<input type="checkbox" name="normalize" id="normalize"<?php
			if ($normalize) echo ' checked="checked"' 
			?> />&nbsp;Normalize XML-well-formed results
		</label>
<?php endif ?>
		<input type="submit" value="Convert" />
	</div>
	</form>

<?php if (isset($_GET['markdown'])) : ?>
<?php		
	# Read input data.
	if (get_magic_quotes_gpc())
		$text = stripslashes($_GET['markdown']);
	else
		$text = $_GET['markdown'];

	# For passing to shell scripts
	$shell_text = escapeshellarg($text);
		
	$result = array();
		
	if ($perl) {
		# Markdown.pl
		$vers = shell_exec("perl Markdown.pl --version");
		$vers = preg_replace('/.*version\s+(.*?)[.]?\s.*/s', '\1', $vers);
		$result = shell_exec("echo $shell_text | perl Markdown.pl 2>&1");
		showResult('Markdown.pl'.$vers, $result);
	
		$vers = shell_exec("perl Markdown-beta.pl --version");
		$vers = preg_replace('/.*version\s+(.*?)[.]?\s.*/s', '\1', $vers);
		$result = shell_exec("echo $shell_text | perl Markdown-beta.pl 2>&1");
		showResult('Markdown.pl'.$vers, $result);
	}
	
	# PHP Markdown & Extra
	include_once 'markdown.php';
	$php_markdown = new Markdown_Parser;
	$php_markdown_extra = new MarkdownExtra_Parser;
	showResult('PHP Markdown '.MARKDOWN_VERSION, $php_markdown->transform($text));
	showResult('PHP Markdown Extra '.MARKDOWNEXTRA_VERSION, $php_markdown_extra->transform($text));
	
	if ($python) {
		# Python Markdown
		$result = shell_exec("echo $shell_text | python stub.py 2>&1");
		showResult('Python Markdown', $result);
	}

	if ($perl) {
		# Text::Markdown
		$script = escapeshellarg('
			use Text::Markdown \'markdown\';
			local $/; # Slurp the whole file
			my $text = <>;
			print markdown($text);
			');
		$result = shell_exec("echo $shell_text | perl -e $script 2>&1");
		showResult('Text::Markdown', $result);
	
		# Text::MultiMarkdown
		$script = escapeshellarg('
			use Text::MultiMarkdown \'markdown\';
			local $/; # Slurp the whole file
			my $text = <>;
			print markdown($text);
			');
		$result = shell_exec("echo $shell_text | perl -e $script 2>&1");
		showResult('Text::MultiMarkdown', $result);
	
		# MultiMarkdown
		$script_file = 'MultiMarkdown/bin/MultiMarkdown.pl';
		$result = shell_exec("echo $shell_text | perl $script_file 2>&1");
		showResult('MultiMarkdown', $result);
	}
	
	if ($ruby) {
		# BlueCloth
		$script = escapeshellarg('
			require \'bluecloth\'
			doc = BlueCloth.new($stdin.read)
			$stdout.write(doc.to_html)
			');
		$result = shell_exec("echo $shell_text | ruby -e $script 2>&1");
		showResult('BlueCloth', $result);
		
		# Maruku
		$script = escapeshellarg('
			require \'maruku\'
			doc = Maruku.new($stdin.read)
			$stdout.write(doc.to_html)
			');
		$result = shell_exec("echo $shell_text | ruby -e $script 2>&1");
		showResult('Maruku', $result);
	}
	
	if ($java) {
		# MarkdownJ
		$jargs = '-classpath ./markdownj.jar';
		$jclass = 'com.petebevin.markdown.MarkdownProcessor';
		$result = shell_exec("echo $shell_text | java $jargs $jclass 2>&1");
		showResult('MarkdownJ', $result);
	}
	
	if ($lua) {
		# MarkdownJ
		$lua = $is_local ? './lua5.1' : './linux/lua5.1';
		$result = shell_exec("echo $shell_text | $lua markdown.lua 2>&1");
		$result = preg_replace('{^.*<body>|</body>.*$}s', '', $result);
		showResult('markdown.lua', $result);
	}
	
	if ($pandoc) {
		# Pandoc
		$script = $is_local ? './pandoc' : 'linux/pandoc';
		$result = shell_exec("echo $shell_text | ./pandoc 2>&1");
		showResult('Pandoc', $result);
	}
	
	# Showdown
	showResult('Showdown', '', 'showdown-result', '
<script type="text/javascript" src="showdown.js"></script>
<div id="showdown-result"></div>
<script type="text/javascript">
	var input = document.getElementById("markdown");
	var converter = new Showdown.converter();
	var html = converter.makeHtml(input.value);
	var result = document.getElementById("showdown-result");
	result.appendChild(document.createTextNode(html));
</script>
<noscript>-- Showdown requires JavaScript --</noscript>
');
?>
<?php endif; /* $_GET['markdown'] is set */ ?>
</div>

<div id="cheatset">
<h2>Featured Implementations</h2>

<?php if ($perl) : ?>
<h3><a href="http://daringfireball.net/projects/markdown">Markdown.pl</a></h3>
<p>First Markdown implementation by Markdown's creator, John Gruber. Written in Perl.</p>
<?php endif ?>

<h3><a href="http://michelf.com/projects/php-markdown">PHP Markdown & Extra</a></h3>
<p>PHP port of Markdown.pl by Michel Fortin. PHP Markdown second flavor, “Extra”, adds some new language constructs.</p>

<?php if ($python) : ?>
<h3><a href="http://www.freewisdom.org/projects/python-markdown/">Python Markdown</a></h3>
<p>Python implementation of Markdown mostly by Yuri Takhteyev and Manfred Stienstra.</p>
<?php endif ?>

<?php if ($perl) : ?>
<h3><a href="http://search.cpan.org/~bobtfish/Text-Markdown-1.0.17/lib/Text/Markdown.pm">Text::Markdown</a></h3>
<p>Perl module based on MultiMarkdown but stripped from the features not present the original Markdown.</p>

<h3><a href="http://search.cpan.org/~bobtfish/Text-Markdown-1.0.17/lib/Text/MultiMarkdown.pm">Text::MultiMarkdown</a></h3>
<p>Perl module for MultiMarkdown.</p>

<h3><a href="http://fletcherpenney.net/MultiMarkdown">MultiMarkdown</a></h3>
<p>A derivative of the original Markdown.pl with added features by Fletcher T. Penney.</p>
<?php endif ?>

<?php if ($ruby) : ?>
<h3><a href="http://www.deveiate.org/projects/BlueCloth">BlueCloth</a></h3>
<p>Ruby implementation of Markdown.</p>

<h3><a href="http://maruku.rubyforge.org/">Maruku</a></h3>
<p>Andrea Censi's Markdown parser with additional features, some borrowed from Markdown Extra.</p>
<?php endif /* ruby */ ?>

<?php if ($java) : ?>
<h3><a href="http://sourceforge.net/projects/markdownj">MarkdownJ</a></h3>
<p>Java implementation of Markdown.</p>
<?php endif /* java */ ?>

<?php if ($lua) : ?>
<h3><a href="http://www.frykholm.se/files/markdown.lua">markdown.lua</a></h3>
<p>Lua implementation by Niklas Frykholm.</p>
<?php endif /* lua */ ?>

<?php if ($pandoc) : ?>
<h3><a href="http://johnmacfarlane.net/pandoc/">Pandoc</a></h3>
<p>Haskell library capable of converting Markdown text to various formats, by John MacFarlane.</p>
<?php endif /* pandoc */ ?>

<h3><a href="http://www.attacklab.net/showdown-gui.html">Showdown</a></h3>
<p>JavaScript port of Markdown.pl, by John Fraser.</p>

<hr />

<p>Babelmark<br />Copyright © 2008 by Michel Fortin</p>
<p>Based on the Markdown Dingus<br />Copyright © 2004 by John Gruber</p>

</div>

</body>
</html>
<?php

function showResult($imp_name, $html, $id = null, $extra_markup = "") {
	global $normalize;
	if ($normalize) $html = normalizeResult($html);
?>
<div>
	<label><?php echo $imp_name ?>:</label><br />
	<pre><code<?php if (isset($id)) echo ' id="'.$id.'"' ?>><?php 
		echo htmlentities(trim($html), ENT_NOQUOTES, 'UTF-8');
	?></code></pre>
<?php echo $extra_markup ?>
</div>
<?php
	flush();
}

function normalizeResult($result) {
	$doc_result = @DOMDocument::loadXML("<!DOCTYPE html>".
		"<html xmlns='http://www.w3.org/1999/xhtml'>".
		"<body>$result</body></html>");

	if ($doc_result) {
		normalizeElementContent($doc_result->documentElement, false);
		$result = $doc_result->saveXML();
		$result = preg_replace('{^.*?<body>|</body>.*?$|^.*?<body\s*/>.*$}is', '', $result);
	} else {
		// couldn't normalize, return in raw form
	}
	return $result;
}



function normalizeElementContent($element, $whitespace_preserve) {
#
# Normalize content of HTML DOM $element. The $whitespace_preserve 
# argument indicates that whitespace is significant and shouldn't be 
# normalized; it should be used for the content of certain elements like
# <pre> or <script>.
#
	$node_list = $element->childNodes;
	switch (strtolower($element->nodeName)) {
		case 'body':
		case 'div':
		case 'blockquote':
		case 'ul':
		case 'ol':
		case 'dl':
		case 'h1':
		case 'h2':
		case 'h3':
		case 'h4':
		case 'h5':
		case 'h6':
			$whitespace = "\n\n";
			break;
			
		case 'table':
			$whitespace = "\n";
			break;
		
		case 'pre':
		case 'script':
		case 'style':
		case 'title':
			$whitespace_preserve = true;
			$whitespace = "";
			break;
		
		default:
			$whitespace = "";
			break;
	}
	foreach ($node_list as $node) {
		switch ($node->nodeType) {
			case XML_ELEMENT_NODE:
				normalizeElementContent($node, $whitespace_preserve);
				normalizeElementAttributes($node);
				
				switch (strtolower($node->nodeName)) {
					case 'p':
					case 'div':
					case 'hr':
					case 'blockquote':
					case 'ul':
					case 'ol':
					case 'dl':
					case 'li':
					case 'address':
					case 'table':
					case 'dd':
					case 'pre':
					case 'h1':
					case 'h2':
					case 'h3':
					case 'h4':
					case 'h5':
					case 'h6':
						$whitespace = "\n\n";
						break;
					
					case 'tr':
					case 'td':
					case 'dt':
						$whitespace = "\n";
						break;
					
					default:
						$whitespace = "";
						break;
				}
				
				if (($whitespace == "\n\n" || $whitespace == "\n") &&
					$node->nextSibling && 
					$node->nextSibling->nodeType != XML_TEXT_NODE)
				{
					$element->insertBefore(new DOMText($whitespace), $node->nextSibling);
				}
				break;
				
			case XML_TEXT_NODE:
				if (!$whitespace_preserve) {
					if (trim($node->data) == "") {
						$node->data = $whitespace;
					} else {
						$node->data = preg_replace('{\s+}', ' ', $node->data);
					}
				}
				break;
		}
	}
	if (!$whitespace_preserve && 
		($whitespace == "\n\n" || $whitespace == "\n"))
	{
		if ($element->firstChild) {
			if ($element->firstChild->nodeType == XML_TEXT_NODE) {
				$element->firstChild->data = 
					preg_replace('{^\s+}', "\n", $element->firstChild->data);
			} else {
				$element->insertBefore(new DOMText("\n"), $element->firstChild);
			}
		}
		if ($element->lastChild) {
			if ($element->lastChild->nodeType == XML_TEXT_NODE) {
				$element->lastChild->data = 
					preg_replace('{\s+$}', "\n", $element->lastChild->data);
			} else {
				$element->insertBefore(new DOMText("\n"), null);
			}
		}
	}
}

function normalizeElementAttributes($element) {
#
# Sort attributes by name.
#
	// Gather the list of attributes as an array.
	$attr_list = array();
	foreach ($element->attributes as $attr_node) {
		$attr_list[$attr_node->name] = $attr_node;
	}
	
	// Sort attribute list by name.
	ksort($attr_list);

	// Remove then put back each attribute following sort order.
	foreach ($attr_list as $attr_node) {
		$element->removeAttributeNode($attr_node);
		$element->setAttributeNode($attr_node);
	}
}

?>